<?php
/**
 * Product Price by Formula for WooCommerce - Default Formula Section Settings
 *
 * @version 2.1.2
 * @since   2.0.0
 * @author  Algoritmika Ltd.
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_PPBF_Settings_Default_Formula' ) ) :

class Alg_WC_PPBF_Settings_Default_Formula extends Alg_WC_PPBF_Settings_Section {

	/**
	 * Constructor.
	 *
	 * @version 2.0.0
	 * @since   2.0.0
	 */
	function __construct() {
		$this->id   = 'default_formula';
		$this->desc = __( 'Default Formula', 'product-price-by-formula-for-woocommerce' );
		parent::__construct();
	}

	/**
	 * get_settings.
	 *
	 * @version 2.1.2
	 * @since   2.0.0
	 */
	function get_settings() {
		$link_start = apply_filters( 'alg_wc_ppbf', '<a href="https://wpfactory.com/item/product-price-by-formula-for-woocommerce/" target="_blank" title="' .
			__( 'To use this shortcode you\'ll need Product Price by Formula for WooCommerce Pro plugin.', 'product-price-by-formula-for-woocommerce' ) . '">', 'settings' );
		$link_end   = apply_filters( 'alg_wc_ppbf', '</a>', 'settings' );
		$default_settings = array(
			array(
				'title'    => __( 'Default Formula Settings', 'product-price-by-formula-for-woocommerce' ),
				'type'     => 'title',
				'desc'     => '<p>' . __( 'You can set default settings here. All settings can later be changed on individual product\'s edit page (in <strong>Product Price by Formula</strong> meta box).', 'product-price-by-formula-for-woocommerce' ) . '</p>' .
					'<p>' . sprintf( __( 'In <strong>formula</strong> use %s variable for product\'s base price. For example: %s. Please note that you can not use %s or %s <strong>inside other params</strong>.', 'product-price-by-formula-for-woocommerce' ),
						'<code>' . 'x' . '</code>', '<code>' . 'x+p1*p2' . '</code>', '<code>' . 'x' . '</code>', '<code>' . 'pN' . '</code>' ) . '</p>' .
					'<p>' . sprintf( __( 'In <strong>formula and/or params</strong> use can also use shortcodes: %s.', 'product-price-by-formula-for-woocommerce' ), '<code>' . implode( '</code>, <code>', array(
						'[product_total_sales]',
						'[product_stock]',
						'[product_weight]',
						'[product_length]',
						'[product_width]',
						'[product_height]',
						'[product_meta]',
						'[if_customer_location]',
						'[if_time]',
						'[if_date]',
						'[if_user_role]',
						'[math_round]',
						'[math_ceil]',
						'[math_floor]',
						'[math_min]',
						'[math_max]',
						$link_start . '[if_value]'            . $link_end,
						$link_start . '[if_product_category]' . $link_end,
						$link_start . '[if_product_tag]'      . $link_end,
						$link_start . '[if_product_taxonomy]' . $link_end,
					) ). '</code>' ) . '</p>' .
					'<p>' . sprintf( __( 'Please note that if you are using <strong>caching plugins</strong> and dynamic product pricing (e.g.: price changing with product stock (%s) or by customer\'s location (%s)), then caching needs to be disabled for products pages.', 'product-price-by-formula-for-woocommerce' ),
						'<code>[product_stock]</code>', '<code>[if_customer_location]</code>' ) . ' ' .
					sprintf( __( 'If you want to keep caching enabled, you will need to cache product pages for each condition: for example for %s you can set %s option to %s in %s.', 'product-price-by-formula-for-woocommerce' ),
						'<code>[if_customer_location]</code>',
						'<em>' . __( 'Default customer location', 'woocommerce' ) . '</em>',
						'<em>' . __( 'Geolocate (with page caching support)', 'woocommerce' ) . '</em>',
						'<em>' . __( 'WooCommerce > Settings > General', 'product-price-by-formula-for-woocommerce' ) . '</em>' ) . '</p>',
				'id'       => 'alg_wc_ppbf_default_options',
			),
			array(
				'title'    => __( 'Formula', 'product-price-by-formula-for-woocommerce' ),
				'type'     => 'textarea',
				'id'       => 'alg_wc_ppbf_eval',
				'default'  => '',
				'css'      => 'width:100%;',
			),
			array(
				'title'    => __( 'Number of parameters', 'product-price-by-formula-for-woocommerce' ),
				'desc'     => '<button name="save" class="button-primary woocommerce-save-button" type="submit" value="' . esc_attr( __( 'Save changes', 'woocommerce' ) ) . '">' . esc_html( __( 'Save changes', 'woocommerce' ) ) . '</button>',
				'desc_tip' => __( 'Save settings after you change this number - new settings fields will appear.', 'product-price-by-formula-for-woocommerce' ),
				'id'       => 'alg_wc_ppbf_total_params',
				'default'  => 1,
				'type'     => 'number',
				'custom_attributes' => array( 'min' => 0 ),
			),
		);
		for ( $i = 1; $i <= get_option( 'alg_wc_ppbf_total_params', 1 ); $i++ ) {
			$default_settings = array_merge( $default_settings, array(
				array(
					'title'    => 'p' . $i . ( '' != ( $admin_note = get_option( 'alg_wc_ppbf_param_note_' . $i, '' ) ) ? ' (' . $admin_note . ')' : '' ),
					'desc'     => __( 'Value', 'product-price-by-formula-for-woocommerce' ),
					'id'       => 'alg_wc_ppbf_param_' . $i,
					'default'  => '',
					'type'     => 'text',
				),
				array(
					'desc'     => __( 'Admin note (optional)', 'product-price-by-formula-for-woocommerce' ),
					'id'       => 'alg_wc_ppbf_param_note_' . $i,
					'default'  => '',
					'type'     => 'text',
				),
			) );
		}
		$default_settings = array_merge( $default_settings, array(
			array(
				'type'     => 'sectionend',
				'id'       => 'alg_wc_ppbf_default_options',
			),
		) );
		return $default_settings;
	}

}

endif;

return new Alg_WC_PPBF_Settings_Default_Formula();
